/*******************************************************************************
* correctintensity.c
*
* This utility performs an intensity correction
* on a MEIS 2d energy-angle spectrum
*/
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <malloc.h>
#include <float.h>
#include <math.h>

#ifndef WIN32
//#include "macros.h"
#endif

#include "meis.h"
#include "eglib.h"

#define MAX(a, b) ((a) < (b) ? (b) : (a))

enum dimensions {A = 0, E = 1};

int             widths[] = {1, 1, 2, 2, 4, 4, 4};
char           *dir;
char           *fromspec;
char           *tospec;
int             bases[8];
int             ranges[8];
int             layouts[2];
int             types[2];
int             dim;
int             numspecs;
double          maximum[2] = {0.0, 0.0};
double          minimum[2] = {DBL_MAX, DBL_MAX};


void readfailed ()
{
	char *buf;
	buf = EGerrorMessage (EGerrno);
	printf ("%s: %s\n", buf, fromspec);
	exit (1);
}

void writefailed ()
{
	char *buf;
	buf = EGerrorMessage (EGerrno);
	printf ("%s: %s\n", buf, tospec);
	exit (1);
}

void notimplemented (char *buf)
{
	printf ("%s: %s\n", "not implemented", buf);
	exit (1);
}

void EGcrash ()
{
	char *buf;
	buf = EGerrorMessage (EGerrno);
	printf ("%s\n", buf);
	exit (1);
}

void crash (char *buf)
{
	printf ("%s\n", buf);
	exit (1);
}
/*******************************************************************************
*/
/*******************************************************************************
*	argv[1]: input spectrum
*	argv[2]: output spectrum
*	argv[3]: optional corrections file
*/
int main (int argc, char *argv[])
{
	int    i, n, rangeEnergy;
	char   buf[256];
	int    *counts;                   /* buffers for performing correction */
	float correction, *corrections, *pCorrections, tmpCorr, range2;
	char *caps = "capabilities";
	FILE *cp;

	dir = (char *) getenv ("EXPERIMENTDIR");
	if (dir==NULL || *dir=='\0') dir = caps = "";

	if (EGsetExperimentPath (dir))
		EGcrash ();
	if (EGconnectExperiment (caps))
		EGcrash ();

	if (argc != 3 && argc != 4)
	{
		printf ("Usage: correctintensity from_spec to_spec [correction_file]\n");
		exit (1);
	}
	fromspec = argv[1];
	tospec = argv[2];

	if (EGinquireSpectrum (fromspec, &dim, bases, ranges, layouts, types))
		readfailed ();
	if (dim != 2)
		notimplemented ("spectrum not two dimensional");

	if (EGcreate2dSpectrum (tospec,bases[0],ranges[0],bases[1],ranges[1],layouts[0],types[0]))
		writefailed ();

	rangeEnergy = MAX(ranges[E], DATA_ENERGY);
	corrections = malloc (rangeEnergy * sizeof(float));
	range2 = (float) ranges[E] / (float) 2.0;

	for (i = bases[E], pCorrections = corrections; i < bases[E]+ranges[E]; i++, pCorrections++)
	{
		*pCorrections = (float)(1.0 / (1.0 - 0.000021 * ((float)i - range2) * ((float)i - range2)));
	}
	if (argc == 4) {
		cp = fopen(argv[3], "r");
		if (!cp) {
			char message[80];
			sprintf(message, "Couldn't open corrections file \"%s\"",argv[3]);
			perror(message);
			printf("Using calculated correction values.\n");
		} else {
			for (i=0, pCorrections = corrections; i<rangeEnergy; i++, pCorrections++) {
				n = fscanf(cp, "%g", &tmpCorr);
				if (n == 1) *pCorrections = tmpCorr;
				if (n == EOF) break;
			}
			fclose (cp);
		}
	}

	counts  = malloc (ranges[A] * widths[types[0]]);

	for (i = bases[E], pCorrections = corrections; i < bases[E]+ranges[E]; i++, pCorrections++)
	{
		if (EGread2dSpectrum (fromspec, bases[A], ranges[A], i, 1, counts, types[0]))
			readfailed ();

/* printf ("%g\n", (double)*pCorrections);  */
		for (n = 0; n < ranges[A]; n++)
			counts[n] = (int) ((float) counts[n] * *pCorrections + 0.5);

		if (EGwrite2dSpectrum (tospec, bases[A], ranges[A], i, 1, counts, types[0]))
			writefailed ();
	}

	free (counts);
	free (corrections);

	for (n = 1; n <= 16; n++) {
		i = EGreadInformation (fromspec, n, buf);
		if (i != 0)
			readfailed ();
		if (*buf != '\0')
			i = EGwriteInformation (tospec, n, buf);
		if (i != 0)
			writefailed ();
	}
	for (n = 1; n <= dim; n++) {
		i = EGreadAnnotation (fromspec, n, buf);
		if (i != 0)
			readfailed ();
		if (*buf != '\0')
			i = EGwriteAnnotation (tospec, n, buf);
		if (i != 0)
			writefailed ();
	}
	for (n = 1; n <= dim; n++) {
		i = EGreadCalibration (fromspec, n, buf);
		if (i != 0)
			readfailed ();
		if (*buf != '\0')
			i = EGwriteCalibration (tospec, n, buf);
		if (i != 0)
			writefailed ();
	}
	for (n = 1; n <= dim; n++) {
		i = EGreadEfficiency (fromspec, n, buf);
		if (i != 0)
			readfailed ();
		if (*buf != '\0')
			i = EGwriteEfficiency (tospec, n, buf);
		if (i != 0)
			writefailed ();
	}

	printf ("spectrum %s corrected for intensity to %s\n", fromspec, tospec);
	return 0;
}
/*******************************************************************************
*/
