/*	The code in this file writes a sample Igor Pro packed experiment file.
	
	See Igor Pro Tech Note PTN#003 for details.
*/

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <stddef.h>					// For offsetof macro.

#include "IgorBin.h"
#include "CrossPlatformFileIO.h"

#include "WaveGlue.h"


static void* waveDataPtr = NULL;
static int waveType = -1;
static int waveWidth = -1;
static int waveHeight = -1;
static char waveName[64];

static void
PrintResult(const char* message, int error)
{
	char buffer[256];
	
	if (error)
		sprintf(buffer, "Error %d: %s", error, message);
	else
		strcpy(buffer, message);
	printf("%s\n", buffer);
}

static void
ReorderBytes(void *p, int bytesPerPoint, long numValues)	// Reverses byte order.
{
	unsigned char ch, *p1, *p2, *pEnd;
	
	pEnd = (unsigned char *)p + numValues*bytesPerPoint;
	while (p < (void *)pEnd) {
		p1 = p;
		p2 = (unsigned char *)p + bytesPerPoint-1;
		while (p1 < p2) {
			ch = *p1;
			*p1++ = *p2;
			*p2-- = ch;
		}
		p = (unsigned char *)p + bytesPerPoint;
	}
}

static void
ReorderShort(void* sp)
{
	ReorderBytes(sp, 2, 1);
}

static void
ReorderLong(void* lp)
{
	ReorderBytes(lp, 4, 1);
}

static void
ReorderDouble(void* dp)
{
	ReorderBytes(dp, 8, 1);
}

static int
ReadWaveRecord(CP_FILE_REF fr)
{
//	unsigned long numBytesToRead, numBytesRead;
//	unsigned short stemp;
//	int needToReorderBytes;

	int type, width, height;
	long npnts;
	int err;
	
	/*	First we must determine if the file uses the wrong byte ordering, in which
		case we will need to do reordering. We do this by reading the version
		field of the binHeader.
	*/
	/*
	numBytesToRead = 2;
	if (err = CPReadFile(fr, numBytesToRead, &stemp, &numBytesRead)) {
		PrintResult("Read file error", err);
		return err;
	}
	needToReorderBytes = 0;
	if ((stemp & 0xFF00) != 0)					// High byte is non-zero.
		needToReorderBytes = 1;					// The file uses the wrong byte ordering.	
	CPSetFilePosition(fr, -2, 0);				// Back up to the starting point.

if (needToReorderBytes) {
		#if defined(__i386__) || defined(_M_IX86)
			printf("This file uses big-endian byte ordering.\n");
		#else
			printf("This file uses little-endian byte ordering.\n");
		#endif
	}
*/

	err = ReadWave(fr, &type, &npnts, &waveDataPtr, &width, &height, waveName);
	waveType = type;
	waveWidth = width;
	waveHeight = height;
	
	// If no error occurred, the main wave data is now available via waveDataPtr.

	if (err) {
		PrintResult("Error reading wave", err);
		return err;
	}

	return 0;
}

int
ReadWaveFile(const char* waveFilePath)
{
	CP_FILE_REF fr;			// Ref number for the file we are writing to.
//	unsigned long numberOfBytesInFile;
//	char temp[256];
	int err;
	
	if (err = CPOpenFile(waveFilePath, 0, &fr)) {
		PrintResult("Open file error", err);
		return err;
	}
	
	// Read the wave data
	if (err = ReadWaveRecord(fr))
		return err;
	
	if (err = CPCloseFile(fr)) {
		PrintResult("Close file error", err);
		return err;
	}
	
//	sprintf(temp, " \"%s\" read successfully.", waveFilePath);
//	PrintResult(temp, 0);
	
	return 0;
}


/* Convert from Igor types to MIDAS types */
int
getWaveType()
{
	int midasType;
	
	if (waveType & NT_CMPLX)
		return -1;			// Complex wave - not supported.
	
	if (waveType & NT_UNSIGNED)
	// Consider the number type, not including the complex bit or the unsigned bit.
	switch(waveType & ~(NT_CMPLX | NT_UNSIGNED)) {
		case NT_I8:
			midasType = 0;		// char
			break;
		case NT_I16:
			midasType = 2;		// short
			break;
		case NT_I32:
			midasType = 4;		// long
			break;
		case NT_FP32:
			midasType = 6;		// float
			break;
		case NT_FP64:
			midasType = 7;		// double (not actually in MIDAS)
			break;
		default:
			return -1;
			break;
	}
	else
	switch(waveType & ~(NT_CMPLX | NT_UNSIGNED)) {
		case NT_I8:
			midasType = 1;		// char
			break;
		case NT_I16:
			midasType = 3;		// short
			break;
		case NT_I32:
			midasType = 5;		// long
			break;
		case NT_FP32:
			midasType = 6;		// float
			break;
		case NT_FP64:
			midasType = 7;		// double (not actually in MIDAS)
			break;
		default:
			return -1;
			break;
	}

	return midasType;
}

int
getBytesPerPoint()
{
	int numBytesPerPoint;
	
	if (waveType & NT_CMPLX)
		return -1;			// Complex wave - not supported.
	
	// Consider the number type, not including the complex bit or the unsigned bit.
	switch(waveType & ~(NT_CMPLX | NT_UNSIGNED)) {
		case NT_I8:
			numBytesPerPoint = 1;		// char
			break;
		case NT_I16:
			numBytesPerPoint = 2;		// short
			break;
		case NT_I32:
			numBytesPerPoint = 4;		// long
			break;
		case NT_FP32:
			numBytesPerPoint = 4;		// float
			break;
		case NT_FP64:
			numBytesPerPoint = 8;		// double
			break;
		default:
			return -1;
			break;
	}

	return numBytesPerPoint;
}

/* Convert from Igor data to MIDAS signed integer */
/* Assumes data can be converted */
int
getWaveData(void *ptr)
{
	int midasData;
	
	if (waveType & NT_CMPLX)
		return 0;			// Complex wave - not supported.
	
	if (waveType & NT_UNSIGNED)
	// Consider the number type, not including the complex bit or the unsigned bit.
	switch(waveType & ~(NT_CMPLX | NT_UNSIGNED)) {
		case NT_I8:
			midasData = (int) *(char *)ptr;
			break;
		case NT_I16:
			midasData = (int) *(short *)ptr;
			break;
		case NT_I32:
			midasData = (int) *(int *)ptr;
			break;
		case NT_FP32:
			midasData = (int) *(float *)ptr;
			break;
		case NT_FP64:
			midasData = (int) *(double *)ptr;
			break;
		default:
			return 0;
			break;
	}
	else
	switch(waveType & ~(NT_CMPLX | NT_UNSIGNED)) {
		case NT_I8:
			midasData = (int) *(unsigned char *)ptr;
			break;
		case NT_I16:
			midasData = (int) *(unsigned short *)ptr;
			break;
		case NT_I32:
			midasData = (int) *(unsigned int *)ptr;
			break;
		case NT_FP32:
			midasData = (int) *(float *)ptr;
			break;
		case NT_FP64:
			midasData = (int) *(double *)ptr;
			break;
		default:
			return 0;
			break;
	}

	return midasData;
}

int
getWaveWidth() { return waveWidth; }

int
getWaveHeight() { return waveHeight; }

int
getWaveDimension() { return (waveHeight==-1) ? -1 : (waveHeight!=0) ? 2 : 1; }

char *
getWaveName() { return &waveName[0]; }

void *
getWaveDataPtr() { return waveDataPtr; }

void
freeWaveDataPtr()
{
	if (waveDataPtr != NULL)
		free(waveDataPtr);
}
