/*******************************************************************************
* ksquared.c
*
* This utility performs a ksquared correction 
* on a MEIS 2d energy-angle spectrum
* the result should be a spectrum normalised to 90' scattering/recoils
*/
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <eglib.h>
#include <malloc.h>
#include <math.h>
#include <ctype.h>
#include <float.h>

#ifndef WIN32
#include <macros.h>
#endif

#ifndef PI
#ifndef M_PI
#define M_PI		3.14159265358979323846
#endif
#define PI  M_PI		/* defined in math.h */
#endif

/* extern int      EGerrno; */

/* int A=0, E=1; */
enum dimensions {A = 0, E = 1};
enum algorithms {R = 0, S = 1};

int             widths[] = {1, 1, 2, 2, 4, 4, 4};
char           *dir;
char           *fromspec;
char           *tospec;
int             bases[8];
int             ranges[8];
int             layouts[2];
int             types[2];
int             dim;
int             numspecs;
double          maximum[2] = {0.0, 0.0};
double          minimum[2] = {DBL_MAX, DBL_MAX};


void readfailed ()
{
	char *buf;
	buf = EGerrorMessage (EGerrno);
	printf ("%s: %s\n", buf, fromspec);
	exit (1);
}

void writefailed ()
{
	char *buf;
	buf = EGerrorMessage (EGerrno);
	printf ("%s: %s\n", buf, tospec);
	exit (1);
}

void notimplemented (char *buf)
{
	printf ("%s: %s\n", "not implemented", buf);
	exit (1);
}

void EGcrash ()
{
	char *buf;
	buf = EGerrorMessage (EGerrno);
	printf ("%s\n", buf);
	exit (1);
}

void crash (char *buf)
{
	printf ("%s\n", buf);
	exit (1);
}
/*******************************************************************************
* kSquared return ksqured function
*/
double kSquared (double m1, double m2, double angle)
{
	double k, m1sin;
	angle *= M_PI / 180.0;                          /* convert to radians */
	m1sin = m1 * sin (angle);
	k = (sqrt (m2*m2 - m1sin*m1sin) + m1 * cos (angle)) / (m1 + m2);

	return (k * k);
}

double kSquaredS (double m1, double m2, double angle)
{
	double k, a, A;
	angle *= M_PI / 180.0;                          /* convert to radians */
	A = m2/m1;
	a = cos(angle) + sqrt((A*A)-sin(angle)*sin(angle));
	k = a / (1.0 + A);

	return (k * k);
}

double kSquaredR (double m1, double m2, double angle)
{
	double k, A;
	angle *= M_PI / 180.0;                          /* convert to radians */
	A = m2/m1;
	k = cos(angle) / (1.0 + A);

	return (4.0 * A * k * k);
}
/*******************************************************************************
* return nearest integer to product of count and overlap
*/
int proportion (int count, double scale, double overlap)
{
	return (int) ((double) count * scale * overlap + 0.5);
}

/*******************************************************************************
*/
int *counts, *correct;                   /* buffers for performing correction */
double scale;
int   bA, bE, rA, rE;                                       /* size of window */
int   rEspec;                  /* Energy range to be written to erda spectrum */
int   bEspec;                                  /* Base of energy to be written */
double m[2], c[2];                                   /* calibration constants */
/*******************************************************************************
* correct one collumn of data
*/
void colCorrect (double ksq)
{
	int    i, j;
	double energy, k2emax, k2emin, emax, emin;
	double hi, lo;

	memset (correct, 0, rE * widths[types[0]]);         /* initialise memory */

	for (i = j = 0; i < rE; i++)            /* step through corrected buffer */
	{
		if (j)
			j--;
		energy = m[E] * i + c[E];
		k2emin = ksq * energy;
		k2emax = ksq * (energy + m[E]);

		for (; j < ranges[E]; j++)         /* extract counts from raw buffer */
		{
			emin = m[E] * j + c[E];
			emax = emin + m[E];

			if (emin >= k2emax)   /* then bin complete - move on to next one */ 
				break;

			else if (emax <= k2emin)   /* then no overlap - fetch next count */
				continue;

			hi = min (emax, k2emax);
			lo = max (emin, k2emin);
			correct[i] += proportion (counts[j], scale, (hi - lo) / (emax - emin));
/*
			{
			int p = proportion (counts[j], (hi - lo) / (emax - emin));
			correct[i] += p;
			if (p>1)
				continue;
			}
*/
		}
	}
}
/*******************************************************************************
*	argv[1]: input spectrum
*	argv[2]: output spectrum
*	argv[3]: E0 - initial energy
*	argv[4]: m1 - mass of beam nuclius
*	argv[5]: m2 - mass of target nuclius
*/
int main (int argc, char *argv[])
{
	int    i, n, alg;
	char   buf[256];
	char   type[20], unit[20];
	double E0, m1, m2;
	double angle, ksq, energy, offset;
	char *caps = "capabilities";

	alg = S;
	scale = 1.0;

	dir = (char *) getenv ("EXPERIMENTDIR");
	if (dir==NULL || *dir=='\0') dir = caps = "";

	if (EGsetExperimentPath (dir))
		EGcrash ();
	if (EGconnectExperiment (caps))
		EGcrash ();

	if (argc > 1 && strlen(argv[1]) == 1) {
		char c = tolower(*argv[1]);
		if (c=='e' || c=='r' || c=='m' || c=='s') {
			if (c=='e' || c=='r')
				alg = R;
			else if (c=='m' || c=='s')
				alg = S;
			argc--; argv++;
		}
	}

	if (argc > 1 && strlen(argv[1]) == 1) {
		char c = tolower(*argv[1]);
		if (c=='k') {
			scale = 1000.0;
			argc--; argv++;
		}
	}

	if (argc != 6 && argc != 7)
	{
		printf ("Usage: ksquared <e|r | m|s> <k> from_spec to_spec beam_energy beam_mass target_mass <angle offset>\n");
		exit (1);
	}
	fromspec = argv[1];
	tospec = argv[2];
	sscanf (argv[3], "%lf", &E0);
	sscanf (argv[4], "%lf", &m1);
	sscanf (argv[5], "%lf", &m2);
	if (argc == 7)
		sscanf (argv[6], "%lf", &offset);
	else
		offset = 0.0;


	if (EGinquireSpectrum (fromspec, &dim, bases, ranges, layouts, types))
		readfailed ();
	if (dim != 2)
		notimplemented ("spectrum not two dimensional");

	for (i = 0; i < dim; i++)                  /* get calibration strings */
	{
		if (EGreadCalibration (fromspec, i + 1, buf))
			readfailed (fromspec);
		if (*buf != '\0')
			sscanf (buf, "%s %s %d %lf %lf", type, unit, &n, &c[i], &m[i]);
	}
	c[A] += offset;

	bE = bases[E];
	rE = ranges[E];

	energy = m[E] * (bE + rE) + c[E];
	if (energy < E0)          /* then increase range of output spectrum */
		rE += (int) ((E0 - energy) / m[E] + 20);

/* For a recoil (erda) calc we only write out the top 20% of the spectrum energy */
/* rest of the spectrum is empty/uninteresting */
/* Don't forget to adjust the offset (constant) for the energy calibration, later on */
	bEspec = alg==R ? (int)(0.8*rE) : 0;
	rEspec = rE - bEspec;

	if (EGcreate2dSpectrum (tospec,bases[0],ranges[0],bE,rEspec,layouts[0],types[0]))
		writefailed ();

	counts  = malloc (ranges[E] * widths[types[0]]);
	correct = malloc (rE        * widths[types[0]]);

	for (i = bases[A]; i < bases[A]+ranges[A]; i++)
	{
		angle = m[A] * i + c[A];
		ksq   = alg==R ? kSquaredR (m1, m2, angle) :
				 kSquared  (m1, m2, angle);

		if (EGread2dSpectrum (fromspec, i, 1, bases[E], ranges[E], counts, types[0]))
			readfailed ();

		colCorrect (ksq);

		if (EGwrite2dSpectrum (tospec, i, 1, bE, rEspec, correct+bEspec, types[0]))
			writefailed ();
	}

	free (counts);
	free (correct);

	for (n = 1; n <= 16; n++) {
		i = EGreadInformation (fromspec, n, buf);
		if (i != 0)
			readfailed ();
		if (*buf != '\0') {
			if (n==3 && offset!=0.0) {
				sprintf(buf, "%s {Offset: \"%lf deg\"}", buf, offset);
			}
			i = EGwriteInformation (tospec, n, buf);
			if (i != 0)
				writefailed ();
		}
	}
	for (n = 1; n <= dim; n++) {
		i = EGreadAnnotation (fromspec, n, buf);
		if (i != 0)
			readfailed ();
		if (*buf != '\0') {
			i = EGwriteAnnotation (tospec, n, buf);
		}
		if (i != 0)
			writefailed ();
	}
	for (n = 1; n <= dim; n++) {
		int tn;
		double tc, tm;
		i = EGreadCalibration (fromspec, n, buf);
		if (i != 0)
			readfailed ();

		if (*buf != '\0') {
			if ((n-1) == A) {
				sscanf (buf, "%s %s %d %lf %lf", type, unit, &tn, &tc, &tm);
				tc += offset;
				sprintf (buf, "%s %s %d %lf %lf", type, unit, tn, tc, tm);
			} else 	if ((n-1) == E) {
				sscanf (buf, "%s %s %d %lf %lf", type, unit, &tn, &tc, &tm);
				tc += (tm * bEspec);
				sprintf (buf, "%s %s %d %lf %lf", type, unit, tn, tc, tm);
			}
			i = EGwriteCalibration (tospec, n, buf);
			if (i != 0)
				writefailed ();
		}
	}
	for (n = 1; n <= dim; n++) {
		i = EGreadEfficiency (fromspec, n, buf);
		if (i != 0)
			readfailed ();
		if (*buf != '\0') {
			i = EGwriteEfficiency (tospec, n, buf);
		}
		if (i != 0)
			writefailed ();
	}

	printf ("spectrum %s corrected for k squared to %s (%s)\n", fromspec, tospec, alg==R ? "recoils" : "scattering");
	return 0;

}
/*******************************************************************************
*/
