/*******************************************************************************
* distortspectrum.c
*
* This utility adjust a raw 2d MEIS spectrum for detector distortion
*/
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <eglib.h>
#include <malloc.h>
#include <limits.h>
#include <float.h>
#include <math.h>

#ifndef WIN32
#include <macros.h>
#endif

#include "meis.h"
#include "correction.h" /* for definitions of baseEnergy & serrationEnergy */

enum dimensions {A = 0, E = 1};

int             widths[] = {1, 1, 2, 2, 4, 4, 4};
char           *dir;
char           *fromspec;
char           *tospec;
int             bases[8];
int             ranges[8];
int             layouts[2];
int             types[2];
int             dim;
int             numspecs;
double          maximum[2] = {0.0, 0.0};
double          minimum[2] = {DBL_MAX, DBL_MAX};


void readfailed ()
{
	char *buf;
	buf = EGerrorMessage (EGerrno);
	printf ("%s: %s\n", buf, fromspec);
	exit (1);
}

void writefailed ()
{
	char *buf;
	buf = EGerrorMessage (EGerrno);
	printf ("%s: %s\n", buf, tospec);
	exit (1);
}

void notimplemented (char *buf)
{
	printf ("%s: %s\n", "not implemented", buf);
	exit (1);
}

void EGcrash ()
{
	char *buf;
	buf = EGerrorMessage (EGerrno);
	printf ("%s\n", buf);
	exit (1);
}

void crash (char *buf)
{
	printf ("%s\n", buf);
	exit (1);
}
/*******************************************************************************
*	argv[1]: input spectrum
*	argv[2]: output spectrum
*/
int
main (int argc, char *argv[])
{
	int    i, j, k, n;
	int counts[DATA_ENERGY];
	char   buf[256];
	char   type[20], unit[20];
	double c, m, energy, angle;
	char *caps = "capabilities";
	FILE *cp;

	dir = (char *) getenv ("EXPERIMENTDIR");
	if (dir==NULL || *dir=='\0') dir = caps = "";

	if (EGsetExperimentPath (dir))
		EGcrash ();
	if (EGconnectExperiment (caps))
		EGcrash ();

	if (argc != 4)
	{
		printf ("Usage: %s from_spec to_spec correction_file\n", argv[0]);
		exit (1);
	}
	fromspec = argv[1];
	tospec = argv[2];

	cp = fopen(argv[3], "r");
	if (!cp) {
		perror("Couldn't open corrections file");
		exit(1);
	}
	for (i=0; i<DATA_ANGLE; i++)
		fscanf(cp, "%d", &serrationEnergy[i]);
	fclose (cp);

	if (EGinquireSpectrum (fromspec, &dim, bases, ranges, layouts, types))
		readfailed ();
	if (dim != 2)
		notimplemented ("spectrum not two dimensional");

	if (EGcreate2dSpectrum (tospec,0,DATA_ANGLE,0,DATA_ENERGY,layouts[0],types[0]))
		writefailed ();

	for (i = 0; i < DATA_ANGLE; i++)
	{
		j = i + BASE_ANGLE;
		k = baseEnergy[i] + serrationEnergy[i] + 3;

		if (EGread2dSpectrum (fromspec, j, 1, k, DATA_ENERGY, counts, types[0]))
			readfailed ();

		if (EGwrite2dSpectrum (tospec, i, 1, 0, DATA_ENERGY, counts, types[0]))
			writefailed ();
	}

	for (n = 1; n <= 16; n++) {
		i = EGreadInformation (fromspec, n, buf);
		if (i != 0)
			readfailed ();
		if (*buf != '\0')
			i = EGwriteInformation (tospec, n, buf);
		if (i != 0)
			writefailed ();
	}
	for (n = 1; n <= dim; n++) {
		i = EGreadAnnotation (fromspec, n, buf);
		if (i != 0)
			readfailed ();
		if (*buf != '\0')
			i = EGwriteAnnotation (tospec, n, buf);
		if (i != 0)
			writefailed ();
	}
	for (n = 1; n <= dim; n++) {
		i = EGreadCalibration (fromspec, n, buf);
		if (i != 0)
			readfailed ();
		if (*buf != '\0')
			i = EGwriteCalibration (tospec, n, buf);
		if (i != 0)
			writefailed ();
	}
	for (n = 1; n <= dim; n++) {
		i = EGreadEfficiency (fromspec, n, buf);
		if (i != 0)
			readfailed ();
		if (*buf != '\0')
			i = EGwriteEfficiency (tospec, n, buf);
		if (i != 0)
			writefailed ();
	}

		if (EGreadCalibration (fromspec, 1, buf))
			readfailed ();
		if (*buf != '\0')
		{
			sscanf (buf, "%s %s %d %lf %lf", type, unit, &n, &c, &m);

			angle = m * ranges[0]/2 + c;
			m = 24.0 / (double) DATA_ANGLE;
			c = angle - 12.0;

			sprintf (buf, "%s %s %d %f %f", type, unit, n, c, m);

			if (EGwriteCalibration (tospec, 1, buf))
				writefailed ();
		}

		if (EGreadCalibration (fromspec, 2, buf))
			readfailed ();
		if (*buf != '\0')
		{
			sscanf (buf, "%s %s %d %lf %lf", type, unit, &n, &c, &m);

			energy = m * ranges[0]/2 + c;
			m = energy * 0.02 / (double) DATA_ENERGY;
			c = energy * 0.99;

			sprintf (buf, "%s %s %d %f %f", type, unit, n, c, m);

			if (EGwriteCalibration (tospec, 2, buf))
				writefailed ();
		}

	printf ("spectrum %s corrected for detector distortion to %s\n", fromspec, tospec);
	return (0);

}
/*******************************************************************************
*/
