/*******************************************************************************
* squashspectrum.c
*
* This utility performs a squashes the Energy axis
* on a MEIS 2d energy-angle spectrum
*/
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <eglib.h>
#include <malloc.h>
#include <limits.h>
#include <macros.h>
#include <math.h>
#include <float.h>

enum dimensions {A = 0, E = 1};

int widths[] = {sizeof(unsigned char),
				sizeof(char),
				sizeof(unsigned short),
				sizeof(short),
				sizeof(unsigned long),
				sizeof(long),
				sizeof(float)
				};

char           *dir;
char           *fromspec;
char           *tospec;
int             bases[8];
int             ranges[8];
int             layouts[2];
int             types[2];
int             dim;
int             numspecs;
double          maximum[2] = {0.0, 0.0};
double          minimum[2] = {DBL_MAX, DBL_MAX};


void readfailed ()
{
	char *buf;
	buf = EGerrorMessage (EGerrno);
	printf ("%s: %s\n", buf, fromspec);
	exit (1);
}

void writefailed ()
{
	char *buf;
	buf = EGerrorMessage (EGerrno);
	printf ("%s: %s\n", buf, tospec);
	exit (1);
}

void notimplemented (char *buf)
{
	printf ("%s: %s\n", "not implemented", buf);
	exit (1);
}

void EGcrash ()
{
	char *buf;
	buf = EGerrorMessage (EGerrno);
	printf ("%s\n", buf);
	exit (1);
}

void crash (char *buf)
{
	printf ("%s\n", buf);
	exit (1);
}
/*******************************************************************************
*/
int *counts, *correct;                   /* buffers for performing correction */
int   bA, bE, rA, rE;                                       /* size of window */
double m[2], c[2];                                   /* calibration constants */
/*******************************************************************************
* correct one collumn of data
*/
void colCorrect (int factor)
{
	int    i, j;

	memset (correct, 0, rE * widths[types[0]]);      /* initialise memory */

 /* step through corrected buffer adding counts from uncorrected original */
	for (i = 0; i < rE; i++)            
		for (j = i * factor; j < ((i+1) * factor); j++)
			if (j < ranges[E])
				correct[i] += counts[j];
}
/*******************************************************************************
*	argv[1]: input spectrum
*	argv[2]: output spectrum
*	argv[3]: factor
*/
int
main (int argc, char *argv[])
{
	int    i, n;
	char   buf[256];
	char   type[20], unit[20];
	int    factor = 2;
	char *caps = "capabilities";

	dir = (char *) getenv ("EXPERIMENTDIR");
	if (dir==NULL || *dir=='\0') dir = caps = "";

	if (EGsetExperimentPath (dir))
		EGcrash ();
	if (EGconnectExperiment (caps))
		EGcrash ();

	if (argc < 3)
	{
		printf ("Usage: squashspectrum from_spec to_spec factor\n");
		exit (1);
	}
	fromspec = argv[1];
	tospec = argv[2];
	if (argc == 4)
		sscanf (argv[3], "%i", &factor);


	if (EGinquireSpectrum (fromspec, &dim, bases, ranges, layouts, types))
		readfailed ();
	if (dim != 2)
		notimplemented ("spectrum not two dimensional");

	for (i = 0; i < dim; i++)                  /* get calibration strings */
	{
		if (EGreadCalibration (fromspec, i + 1, buf))
			readfailed (fromspec);
		if (*buf != '\0')
			sscanf (buf, "%s %s %d %lf %lf", type, unit, &n, &c[i], &m[i]);
	}

	bE = bases[E];
	rE = ranges[E] / factor;

	if (EGcreate2dSpectrum (tospec,bases[0],ranges[0],bE,rE,layouts[0],types[0]))
		writefailed ();

	counts  = malloc (ranges[E] * widths[types[0]]);
	correct = malloc (rE        * widths[types[0]]);

	for (i = bases[A]; i < bases[A]+ranges[A]; i++)
	{
		if (EGread2dSpectrum (fromspec, i, 1, bases[E], ranges[E], counts, types[0]))
			readfailed ();

		colCorrect (factor);

		if (EGwrite2dSpectrum (tospec, i, 1, bE, rE, correct, types[0]))
			writefailed ();
	}

	free (counts);
	free (correct);

	for (n = 1; n <= 16; n++) {
		i = EGreadInformation (fromspec, n, buf);
		if (i != 0)
			readfailed ();
		if (*buf != '\0')
			i = EGwriteInformation (tospec, n, buf);
		if (i != 0)
			writefailed ();
	}
	for (n = 1; n <= dim; n++) {
		i = EGreadAnnotation (fromspec, n, buf);
		if (i != 0)
			readfailed ();
		if (*buf != '\0')
			i = EGwriteAnnotation (tospec, n, buf);
		if (i != 0)
			writefailed ();
	}
	for (n = 1; n <= dim; n++) {
		i = EGreadCalibration (fromspec, n, buf);
		if (i != 0)
			readfailed ();
		if (*buf != '\0')
		{
			if (n == (E+1))
			{
				char cc[256], mm[256];
				int lc, lm;
				sscanf (buf, "%s %s %d %s %s", type, unit, &n, cc, mm);
				sscanf (cc, "%lf", &c[i]); lc = strlen(strchr(cc, '.')) - 1;
				sscanf (mm, "%lf", &m[i]); lm = strlen(strchr(mm, '.')) - 1;
				m[i] *= (double)factor;
				sprintf (buf, "%s %s %d %.*f %.*f", type, unit, n, lc, c[i], lm, m[i]);
			}
			i = EGwriteCalibration (tospec, n, buf);
		}
		if (i != 0)
			writefailed ();
	}
	for (n = 1; n <= dim; n++) {
		i = EGreadEfficiency (fromspec, n, buf);
		if (i != 0)
			readfailed ();
		if (*buf != '\0')
			i = EGwriteEfficiency (tospec, n, buf);
		if (i != 0)
			writefailed ();
	}

	printf ("spectrum %s squashed by a factor of %d\n", fromspec, factor);
	return (0);

}
/*******************************************************************************
*/
